@@ -258,6 +258,7 @@ typedef enum {
   tagOPTION,
   tagDIV,
   tagCHXJIF,  
+  tagCHXJRAW,  
   tagNOBR,  
   tagSMALL,  
   tagSTYLE,  
@@ -356,6 +357,8 @@ struct mod_chxj_config {
   char                  *allowed_cookie_domain; /* default is null */
 
   chxj_new_line_type_t  new_line_type;
+
+  char                  *post_log;              /* post log environment name. */
 };
 
 #define IS_COOKIE_STORE_DBM(X)      ((X) == COOKIE_STORE_TYPE_DBM)
@@ -482,6 +485,8 @@ extern char *chxj_node_convert_chxjif_only(
 extern void *rpl_malloc(size_t n);
 #endif
 
+
+#define TO_ADDR(X) ((unsigned int)(apr_size_t)(X))
 #endif
 /*
  * vim:ts=2 et
