@@ -23,6 +23,7 @@
 #include "chxj_encoding.h"
 #include "chxj_buffered_write.h"
 #include "chxj_header_inf.h"
+#include "chxj_conv_z2h.h"
 
 
 #define GET_CHTML20(X) ((chtml20_t*)(X))
@@ -294,6 +295,11 @@ tag_handler chtml20_handler[] = {
     s_chtml20_chxjif_tag,
     NULL,
   },
+  /* tagCHXJRAW */
+  {
+    s_chtml20_chxjif_tag,
+    NULL,
+  },
   /* tagNOBR */
   {
     NULL,
@@ -582,7 +588,7 @@ chxj_chtml20_emoji_only_converter(request_rec *r, device_table *spec, const char
   chtml20 = &__chtml20;
   doc     = &__doc;
 
-  DBG(r, "REQ[%X] start chxj_chtml20_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] start chxj_chtml20_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   memset(doc,     0, sizeof(Doc));
   memset(chtml20, 0, sizeof(chtml20_t));
 
@@ -623,7 +629,7 @@ chxj_chtml20_emoji_only_converter(request_rec *r, device_table *spec, const char
   }
   chtml20->out = chxj_buffered_write_flush(chtml20->out, &doc->buf);
 
-  DBG(r, "REQ[%X] end chxj_chtml20_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] end chxj_chtml20_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   return chtml20->out;
 }
 
@@ -1676,6 +1682,11 @@ s_chtml20_start_input_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   if (value && *value) {
+    if (type && (STRCASEEQ('s','S',"submit",type) || STRCASEEQ('r','R',"reset",type))) {
+      apr_size_t value_len = strlen(value);
+      value = chxj_conv_z2h(r, value, &value_len, chtml20->entryp);
+    }
+
     W_L(" value=\"");
     W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
@@ -3123,6 +3134,7 @@ s_chtml20_text_tag(void *pdoc, Node *child)
   char        one_byte[2];
   int         ii;
   int         tdst_len;
+  apr_size_t  z2h_input_len;
 
   chtml20 = GET_CHTML20(pdoc);
   doc     = chtml20->doc;
@@ -3171,6 +3183,9 @@ s_chtml20_text_tag(void *pdoc, Node *child)
       tdst = qs_out_apr_pstrcat(r, tdst, one_byte, &tdst_len);
     }
   }
+  z2h_input_len = strlen(tdst);
+  tdst = chxj_conv_z2h(r, tdst, &z2h_input_len, chtml20->entryp);
+
   W_V(tdst);
   return chtml20->out;
 }
