@@ -23,6 +23,7 @@
 #include "chxj_qr_code.h"
 #include "chxj_encoding.h"
 #include "chxj_header_inf.h"
+#include "chxj_conv_z2h.h"
 
 #define GET_CHTML50(X) ((chtml50_t *)(X))
 #undef W_L
@@ -292,6 +293,11 @@ tag_handler chtml50_handler[] = {
     s_chtml50_chxjif_tag,
     NULL,
   },
+  /* tagCHXJRAW */
+  {
+    s_chtml50_chxjif_tag,
+    NULL,
+  },
   /* tagNOBR */
   {
     NULL,
@@ -585,7 +591,7 @@ chxj_chtml50_emoji_only_converter(request_rec *r, device_table *spec, const char
   chtml50 = &__chtml50;
   doc     = &__doc;
 
-  DBG(r, "REQ[%X] start chxj_chtml50_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] start chxj_chtml50_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   memset(doc,     0, sizeof(Doc));
   memset(chtml50, 0, sizeof(chtml50_t));
 
@@ -626,7 +632,7 @@ chxj_chtml50_emoji_only_converter(request_rec *r, device_table *spec, const char
   }
   chtml50->out = chxj_buffered_write_flush(chtml50->out, &doc->buf);
 
-  DBG(r, "REQ[%X] end chxj_chtml50_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] end chxj_chtml50_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   return chtml50->out;
 }
 
@@ -1577,6 +1583,11 @@ s_chtml50_start_input_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   if (value && *value) {
+    if (type && (STRCASEEQ('s','S',"submit",type) || STRCASEEQ('r','R',"reset",type))) {
+      apr_size_t value_len = strlen(value);
+      value = chxj_conv_z2h(r, value, &value_len, chtml50->entryp);
+    }
+
     W_L(" value=\"");
     W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
@@ -2916,6 +2927,7 @@ s_chtml50_text_tag(void *pdoc, Node *child)
   char        one_byte[2];
   int         ii;
   int         tdst_len;
+  apr_size_t  z2h_input_len;
 
   chtml50 = GET_CHTML50(pdoc);
   doc     = chtml50->doc;
@@ -2962,6 +2974,8 @@ s_chtml50_text_tag(void *pdoc, Node *child)
       tdst = qs_out_apr_pstrcat(r, tdst, one_byte, &tdst_len);
     }
   }
+  z2h_input_len = strlen(tdst);
+  tdst = chxj_conv_z2h(r, tdst, &z2h_input_len, chtml50->entryp);
   W_V(tdst);
   return chtml50->out;
 }
