@@ -24,6 +24,8 @@
 #include "chxj_str_util.h"
 #include "chxj_header_inf.h"
 #include "chxj_jreserved_tag.h"
+#include "chxj_serf.h"
+#include "chxj_conv_z2h.h"
 
 
 #define GET_JHTML(X) ((jhtml_t *)(X))
@@ -126,6 +128,7 @@ static int   s_jhtml_search_emoji(jhtml_t *jhtml, char *txt, char **rslt);
 
 static char *chxj_istyle_to_mode(apr_pool_t *p, const char *s);
 
+static char *s_add_copyright_parameter(request_rec *r, char *value);
 
 
 tag_handler jhtml_handler[] = {
@@ -299,6 +302,11 @@ tag_handler jhtml_handler[] = {
     s_jhtml_chxjif_tag,
     NULL,
   },
+  /* tagCHXJRAW */
+  {
+    s_jhtml_chxjif_tag,
+    NULL,
+  },
   /* tagNOBR */
   {
     NULL,
@@ -605,7 +613,7 @@ chxj_jhtml_emoji_only_converter(request_rec *r, device_table *spec, const char *
   jhtml = &__jhtml;
   doc   = &__doc;
 
-  DBG(r, "REQ[%X] start chxj_jhtml_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] start chxj_jhtml_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   memset(doc,   0, sizeof(Doc));
   memset(jhtml, 0, sizeof(jhtml_t));
 
@@ -646,7 +654,7 @@ chxj_jhtml_emoji_only_converter(request_rec *r, device_table *spec, const char *
   }
   jhtml->out = chxj_buffered_write_flush(jhtml->out, &doc->buf);
 
-  DBG(r, "REQ[%X] end chxj_jhtml_emoji_eonly_converter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] end chxj_jhtml_emoji_eonly_converter()", (unsigned int)(apr_size_t)r);
   return jhtml->out;
 }
 
@@ -1582,6 +1590,11 @@ s_jhtml_start_input_tag(void *pdoc, Node *node)
     W_L("\"");
   }
   if (value && *value) {
+    if (type && (STRCASEEQ('s','S',"submit",type) || STRCASEEQ('r','R',"reset",type))) {
+      apr_size_t value_len = strlen(value);
+      value = chxj_conv_z2h(r, value, &value_len, jhtml->entryp);
+    }
+
     W_L(" value=\"");
     W_V(chxj_add_slash_to_doublequote(doc->pool, value));
     W_L("\"");
@@ -2092,6 +2105,7 @@ s_jhtml_start_img_tag(void *pdoc, Node *node)
       value = chxj_jreserved_tag_to_safe_for_query_string(r, value, jhtml->entryp);
       value = chxj_add_cookie_parameter(r, value, jhtml->cookie);
       value = chxj_add_cookie_no_update_parameter(r, value);
+      value = s_add_copyright_parameter(r, value);
       W_L(" src=\"");
       W_V(value);
       W_L("\"");
@@ -2101,6 +2115,7 @@ s_jhtml_start_img_tag(void *pdoc, Node *node)
       value = chxj_jreserved_tag_to_safe_for_query_string(r, value, jhtml->entryp);
       value = chxj_add_cookie_parameter(r, value, jhtml->cookie);
       value = chxj_add_cookie_no_update_parameter(r, value);
+      value = s_add_copyright_parameter(r, value);
       W_L(" src=\"");
       W_V(value);
       W_L("\"");
@@ -2575,17 +2590,18 @@ s_jhtml_end_b_tag(void* pdoc, Node* UNUSED(child))
 }
 
 static char*
-s_jhtml_text_tag(void* pdoc, Node* child)
+s_jhtml_text_tag(void *pdoc, Node *child)
 {
-  jhtml_t*     jhtml;
-  Doc*         doc;
-  char*        textval;
-  char*        tmp;
-  char*        tdst;
+  jhtml_t      *jhtml;
+  Doc          *doc;
+  char         *textval;
+  char         *tmp;
+  char         *tdst;
   char         one_byte[2];
   int          ii;
   int          tdst_len;
-  request_rec* r;
+  request_rec  *r;
+  apr_size_t   z2h_input_len;
 
   jhtml = GET_JHTML(pdoc);
   doc   = jhtml->doc;
@@ -2637,6 +2653,9 @@ s_jhtml_text_tag(void* pdoc, Node* child)
       }
     }
   }
+  z2h_input_len = strlen(tdst);
+  tdst = chxj_conv_z2h(r, tdst, &z2h_input_len, jhtml->entryp);
+
   W_V(tdst);
   DBG(r, "end s_jhtml_text_tag()");
   return jhtml->out;
@@ -3441,6 +3460,72 @@ s_jhtml_newline_mark(void *pdoc, Node *UNUSED(node))
   W_NLCODE();
   return jhtml->out;
 }
+
+static char *
+s_add_copyright_parameter(request_rec *r, char *value)
+{
+  request_rec *sub_req = NULL;
+  apr_pool_t *pool;
+  int rr_status;
+  char *content_type = NULL;
+  apr_table_t *headers_out = NULL;
+
+  DBG(r, "REQ[%X] start s_add_copyright_parameter", TO_ADDR(r));
+  apr_pool_create(&pool, r->pool);
+
+  if (chxj_starts_with(value, "http:") || chxj_starts_with(value, "https:")) {
+    apr_table_t *response;
+    int   response_code = 0;
+    response = chxj_serf_head(r, pool, value, &response_code);
+    if (response_code != HTTP_OK) {
+      DBG(r, "REQ[%X] end s_add_copyright_parameter (serf_req->status:[%d])", TO_ADDR(r), response_code);
+      return value;
+    }
+    content_type = (char *)apr_table_get(response, "Content-Type");
+    headers_out = response;
+  }
+  else {
+    //  sub_req = ap_sub_req_method_uri("GET", value, r, r->output_filters);
+    sub_req = ap_sub_req_lookup_uri(value, r, r->output_filters);
+    if (sub_req->status != HTTP_OK) {
+      DBG(r, "REQ[%X] end s_add_copyright_parameter (sub_req->status:[%d])", TO_ADDR(r), sub_req->status);
+      ap_destroy_sub_req(sub_req);
+      return value;
+    }
+    sub_req->header_only = 1;
+    sub_req->main = NULL;
+    rr_status = ap_run_sub_req(sub_req);
+    DBG(r, "REQ[%X] sub_req4:ContentType:[%s]", TO_ADDR(r), sub_req->content_type);
+    DBG(r, "REQ[%X] rr_status:[%d]", TO_ADDR(r), rr_status);
+    content_type = apr_pstrdup(pool, sub_req->content_type);
+    headers_out = sub_req->headers_out;
+  }
+  if (headers_out && apr_table_get(headers_out, "x-jphone-copyright")) {
+    if (content_type && strncasecmp("image/jpeg", content_type, 10) == 0) {
+      if (strchr(value, '?')) {
+        value = apr_psprintf(pool, "%s&_chxj_copy=.jpz", value);
+      }
+      else {
+        value = apr_psprintf(pool, "%s?_chxj_copy=.jpz", value);
+      }
+    }
+    else if (content_type && strncasecmp("image/png", content_type, 9) == 0) {
+      if (strchr(value, '?')) {
+        value = apr_psprintf(pool, "%s&_chxj_copy=.pnz", value);
+      }
+      else {
+        value = apr_psprintf(pool, "%s?_chxj_copy=.pnz", value);
+      }
+    }
+  }
+  if (sub_req) {
+    ap_destroy_sub_req(sub_req);
+  }
+  DBG(r, "REQ[%X] end s_add_copyright_parameter(result:[%s]", TO_ADDR(r), value);
+  return value;
+}
+
+
 /*
  * vim:ts=2 et
  */
