@@ -68,7 +68,6 @@
 #endif
 #include "chxj_serf.h"
 #include "chxj_add_device_env.h"
-#include "chxj_conv_z2h.h"
 #include "chxj_header_inf.h"
 #include "chxj_jreserved_tag.h"
 
@@ -247,7 +246,7 @@ chxj_headers_fixup(request_rec *r)
     break;
   
   default:
-    DBG(r, "REQ[%X] end chxj_headers_fixup() (not mobile)", (unsigned int)(apr_size_t)r);
+    DBG(r, "REQ[%X] end chxj_headers_fixup() (not mobile) spec->device_name[%s] User-Agent:[%s]", (unsigned int)(apr_size_t)r, spec->device_name, user_agent);
     return DECLINED;
 
   }
@@ -262,6 +261,12 @@ chxj_headers_fixup(request_rec *r)
     else {
       char *client_ip = (char *)apr_table_get(r->headers_in, CHXJ_HEADER_ORIG_CLIENT_IP);
       if (client_ip) {
+        if (dconf->post_log) {
+          apr_table_setn(r->subprocess_env, dconf->post_log, dconf->post_log);
+        }
+        else {
+          apr_table_setn(r->subprocess_env, "chxj-post-log", "chxj-post-log");
+        }
         apr_sockaddr_t *address = NULL;
         apr_status_t rv = apr_sockaddr_info_get(&address, ap_get_server_name(r), APR_UNSPEC, ap_get_server_port(r), 0, r->pool);
         if (rv != APR_SUCCESS) {
@@ -448,9 +453,6 @@ chxj_convert(request_rec *r, const char **src, apr_size_t *len, device_table *sp
                                                                 entryp, 
                                                                 cookie);
       }
-      if (dst && *len) {
-        dst = chxj_conv_z2h(r, dst, len, entryp);
-      }
     }
   }
   ap_set_content_length(r, *len);
@@ -478,6 +480,7 @@ chxj_convert(request_rec *r, const char **src, apr_size_t *len, device_table *sp
 static void
 s_convert_guid_parameter_to_header(request_rec *r, const char *param, device_table *spec)
 {
+  DBG(r, "REQ[%X] start s_convert_guid_parameter() param:[%s]", (unsigned int)(apr_size_t)r, param);
   if (strcasecmp(param, "guid") == 0) {
     switch(spec->html_spec_type) {
     case CHXJ_SPEC_XHtml_Mobile_1_0:
@@ -501,6 +504,7 @@ s_convert_guid_parameter_to_header(request_rec *r, const char *param, device_tab
       break;
     }
   }
+  DBG(r, "REQ[%X] end s_convert_guid_parameter()", (unsigned int)(apr_size_t)r);
 }
 
 /**
@@ -579,7 +583,6 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry *entryp, device_tabl
     value = apr_strtok(NULL, "=", &vstate);
     if (! name) continue;
     name = chxj_safe_to_jreserved_tag(r, name);
-
     if (strncasecmp(name, "_chxj", 5) != 0 && strncasecmp(name, "%5Fchxj", sizeof("%5Fchxj")-1) != 0) {
       if (strlen(result) != 0) 
         result = apr_pstrcat(r->pool, result, "&", NULL);
@@ -616,10 +619,13 @@ chxj_convert_input_header(request_rec *r,chxjconvrule_entry *entryp, device_tabl
         result = apr_pstrcat(r->pool, result, dname, "=", dvalue, NULL);
       }
       else {
-        if (strcmp(name, pair_sv) != 0)
+        s_convert_guid_parameter_to_header(r, name, spec);
+        if (strcmp(name, pair_sv) != 0) {
           result = apr_pstrcat(r->pool, result, name, "=", value, NULL);
-        else
+        } 
+        else {
           result = apr_pstrcat(r->pool, result, name, NULL);
+        }
       }
     }
     else
@@ -1226,6 +1232,8 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             if (! ap_is_HTTP_REDIRECT(r->status)) {
               r->status = HTTP_MOVED_TEMPORARILY;
             }
+          }
+          if (ctx->len && ap_is_HTTP_REDIRECT(r->status)) {
             ctx->buffer = apr_pstrdup(pool, "");
             ctx->len    = 0;
             ap_set_content_length(r, (apr_off_t)ctx->len);
@@ -1548,9 +1556,14 @@ chxj_translate_name(request_rec *r)
   DBG(r, "REQ[%X] =======================================================================", (unsigned int)(apr_size_t)r);
   DBG(r, "REQ[%X] ", (unsigned int)(apr_size_t)r);
   DBG(r, "REQ[%X] START REQUEST (uri:[%s] args:[%s])", (unsigned int)(apr_size_t)r, r->unparsed_uri, r->args ? r->args : "");
+  DBG(r, "REQ[%X] METHOD [%s]", TO_ADDR(r), r->method);
   DBG(r, "REQ[%X] ", (unsigned int)(apr_size_t)r);
   DBG(r, "REQ[%X] =======================================================================", (unsigned int)(apr_size_t)r);
+#if 0
   return chxj_trans_name(r);
+#else
+  return DECLINED;
+#endif
 }
 
 
@@ -1565,7 +1578,7 @@ chxj_insert_filter(request_rec *r)
   apr_status_t        rv;
   char                *contentType;
 
-  DBG(r, "REQ[%X] start chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] start chxj_insert_filter()", (unsigned int)(apr_size_t)r);
 
   dconf = chxj_get_module_config(r->per_dir_config, &chxj_module);
 
@@ -1579,21 +1592,21 @@ chxj_insert_filter(request_rec *r)
   if (contentType
       && strncasecmp("multipart/form-data", contentType, 19) == 0) {
     DBG(r, "REQ[%X] detect multipart/form-data ==> no target", (apr_size_t)(unsigned int)r);
-    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (unsigned int)(apr_size_t)r);
     return;
   }
 
   spec = chxj_specified_device(r, user_agent);
   entryp = chxj_apply_convrule(r, dconf->convrules);
   if (!entryp) {
-    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (unsigned int)(apr_size_t)r);
     return;
   }
   ctx = apr_palloc(r->pool, sizeof(*ctx));
   memset(ctx, 0, sizeof(*ctx));
   if ((rv = apr_pool_create(&ctx->pool, r->pool)) != APR_SUCCESS) {
     ERR(r, "%s:%d: failed: new pool create. rv:[%d]", __FILE__,__LINE__,rv);
-    DBG(r, "REQ:[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ:[%X] end chxj_insert_filter()", (unsigned int)(apr_size_t)r);
     return;
   }
   ctx->entryp = entryp;
@@ -1602,8 +1615,8 @@ chxj_insert_filter(request_rec *r)
   ctx->buffer[0] = 0;
 
   if (!entryp || !(entryp->action & CONVRULE_ENGINE_ON_BIT)) {
-    DBG(r,"REQ[%X] EngineOff", (apr_size_t)(unsigned int)r);
-    DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+    DBG(r,"REQ[%X] EngineOff", (unsigned int)(apr_size_t)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter()", (unsigned int)(apr_size_t)r);
     return;
   }
 
@@ -1622,17 +1635,17 @@ chxj_insert_filter(request_rec *r)
     break;
 
   default:
-    DBG(r, "REQ[%X] end chxj_insert_filter() Unknown spec type.", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] end chxj_insert_filter() Unknown spec type(%d).", (unsigned int)(apr_size_t)r, spec->html_spec_type);
     return;
   }
 
 
   if (! apr_table_get(r->headers_in, "X-Chxj-Forward")) {
     ap_add_output_filter("chxj_output_filter", ctx, r, r->connection);
-    DBG(r, "REQ[%X] added Output Filter", (apr_size_t)(unsigned int)r);
+    DBG(r, "REQ[%X] added Output Filter", (unsigned int)(apr_size_t)r);
   }
 
-  DBG(r, "REQ[%X] end chxj_insert_filter()", (apr_size_t)(unsigned int)r);
+  DBG(r, "REQ[%X] end chxj_insert_filter()", (unsigned int)(apr_size_t)r);
 }
 
 
@@ -1704,6 +1717,7 @@ chxj_create_per_dir_config(apr_pool_t *p, char *arg)
   conf->forward_url_base = NULL;
   conf->forward_server_ip = NULL;
   conf->allowed_cookie_domain = NULL;
+  conf->post_log              = NULL;
 
   if (arg == NULL) {
     conf->dir                  = NULL;
@@ -1749,6 +1763,7 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   mrg->forward_url_base = NULL;
   mrg->forward_server_ip = NULL;
   mrg->allowed_cookie_domain = NULL;
+  mrg->post_log         = NULL;
 
   mrg->dir = apr_pstrdup(p, add->dir);
 
@@ -1974,6 +1989,12 @@ chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
   else {
     mrg->allowed_cookie_domain = base->allowed_cookie_domain;
   }
+  if (add->post_log) {
+    mrg->post_log = add->post_log;
+  }
+  else {
+    mrg->post_log = base->post_log;
+  }
   return mrg;
 }
 
@@ -2828,6 +2849,23 @@ cmd_set_new_line_type(
   return NULL;
 }
 
+static const char *
+cmd_post_log_env(
+  cmd_parms   *cmd, 
+  void        *mconfig, 
+  const char  *arg)
+{
+  mod_chxj_config  *dconf;
+  if (strlen(arg) > 255)
+    return "mod_chxj: ChxjPostLogEnv is too long.";
+
+  dconf = (mod_chxj_config *)mconfig;
+
+  dconf->post_log = apr_pstrdup(cmd->pool, arg);
+
+  return NULL;
+}
+
 
 static const command_rec cmds[] = {
   AP_INIT_TAKE1(
@@ -2984,7 +3022,13 @@ static const command_rec cmds[] = {
     NULL,
     OR_ALL,
     "Domain that permits parameter addition for cookie besides hostname.(Default:hostname Only)"),
-  {NULL}
+  AP_INIT_TAKE1(
+    "ChxjPostLogEnv",
+    cmd_post_log_env,
+    NULL,
+    OR_ALL,
+    "for CustomLog directive. mod_chxj's internal POST log environment name.(Default:chxj-post-log)"),
+  {NULL,{NULL},NULL,0,0,NULL},
 };
 
 
